%parse_eos_small_radius_exoplanets_for_fugacity_crisis

%Purpose: Parse Saumon-Chabrier EOS for H2
he_density_factor = 8/7; %enhancement of solar H/He mix density over H density (approximate, for 25% He by mass)

%Control parameters:
Tinit_list = [500,1000,1500,1750];
P_RCB = 1e7; %Pa. 100 bars, order-of-magnitude appropriate for the P_RCB for sub-Neptunes.
Mcore_list = [4:20]; %Earth masses, for the rock+(Fe-metal) core

%Saumon-Chabrier for H2
for neos = 1:2
    
switch neos
    case 1
        d = fopen('H_TAB_I.A');
    case 2
        d = fopen('HE_TAB_I.A');
end

tline = fgetl(d);
tindex = 0;

while ischar(tline)
  tline = fgetl(d);
  if length(tline)<30 & tline>-1  %A new block of good data has started. Reset the counter
     tindex = tindex + 1;
     T(tindex) = str2num(tline(2:5));
     ntoread(tindex) = str2num(tline(8:9));
     for n = 1:ntoread(tindex)
         tline=fgetl(d);
         h(tindex,n,1:11) = sscanf(tline,'%f');
         h(tindex,n,12) = 10.^T(tindex);
     end
  end
end

%Unit conversions:
T = 10.^T;
h(h==0) = NaN;
h(:,:,1) = (10.^h(:,:,1))/10; %dyn/cm^2 --> Pa
h(:,:,4) = (10.^(h(:,:,4)))*1e3; %kg/m^3

fclose(d)

switch neos
    case 1
        HEOS = h;
    case 2
        HEEOS = h;
end

end

%Drop adiabats from an RCB (radiative-convective boundary). We want density
%and temperature as a function of pressure along the adiabat.
%Constant-gravity assumption.


figure
adindex = 1;
for Tinit = Tinit_list; %K (close-in sub-Neptune)
%Figure out what the entropy (S) is for this starting P and T by linear
%interpolation in log P and log T.
Tpane_low = max(find(T<Tinit))
Tpane_high = Tpane_low + 1;

%In the EOS, pressures are the same for each T-pane for low T, although
%higher T panes have additional P rows
p_low = max(find(HEOS(Tpane_low,:,1)<P_RCB))
p_high = p_low + 1;

my_S_Tpane_low = interp1(log10(HEOS(Tpane_low,1:ntoread(Tpane_low),1)), ...
                         HEOS(Tpane_low,1:ntoread(Tpane_low),5),log10(P_RCB));
my_S_Tpane_high = interp1(log10(HEOS(Tpane_high,1:ntoread(Tpane_low),1)), ...
                         HEOS(Tpane_high,1:ntoread(Tpane_low),5),log10(P_RCB));
my_S = interp1(log10([T(Tpane_low),T(Tpane_high)]),[my_S_Tpane_low, my_S_Tpane_high],log10(Tinit))

%Now for each T-pane, interpolate the entropy we found above.
%Note that entropy is already stored as log so linear interpolation is OK
for l = 1:size(HEOS,1)
    p_low = max(find(HEOS(l,1:ntoread(l),5)>my_S)); %increase p, decrease S
    if p_low < size(HEOS,2)
    if isnan(HEOS(l,p_low+1,5)) == 0 %very low T-panes may lack the entropies relevant for close-in exoplanet atmospheres
      my_mix = interp1([HEOS(l,p_low,5),HEOS(l,p_low+1,5)],[0,1],my_S);
      my_adiabat(adindex,l,:) = HEOS(l,p_low,:)*(1-my_mix) + HEOS(l,p_low,:)*my_mix;
    end
    end
end

plot(squeeze(my_adiabat(adindex,:,1)),T(1:size(my_adiabat,2)),'r','LineWidth',adindex)
set(gca,'xscale','log')
set(gca,'yscale','log')
hold on
plot(squeeze(my_adiabat(adindex,:,1)),squeeze(my_adiabat(adindex,:,4)),'k','LineWidth',adindex) %density

%Overplot ideal gas law
V_ideal = 1*8.314*T(1:size(my_adiabat,2))./squeeze(my_adiabat(adindex,:,1));
rho_ideal = 1./V_ideal*0.002; %molecular hydrogen
plot(squeeze(my_adiabat(adindex,:,1)),rho_ideal,'Color','k','LineWidth',adindex,'LineStyle','--')

adindex = adindex + 1;

end

ylabel('red is temperature (K), black is density')
xlabel('Pressure (Pa)');set(gca,'fontsize',14)
grid on

%Figure the radius boost along each adiabat, neglecting
%gravity of the envelope, as a function of base-of-envelope pressure.
G = 6.67e-11; %mks

for Mcore = Mcore_list; %Earth masses
    Mcore
    CoreMassEarths = Mcore;
R_pl = 6378000*CoreMassEarths^0.27; %1 Earth radius
M_pl = 6e24*CoreMassEarths; 

boep_list = [1e8:1e8:1e10]; %base-of-envelope-pressure (Pa);
dr = 1e4; %size of radius increment (m)
%Integrating OUTWARDS from the core-envelope boundary. 
for adindex = 1:size(my_adiabat,1) %RCB adiabatic temperature index
    for l = 1:length(boep_list)
        r = 0; i = 1;
        pr(Mcore,adindex,l,i) = boep_list(l); %base of envelope pressure
        eoa = max(find(my_adiabat(adindex,:,1)>0)); %end of adiabat 
        boa = min(find(my_adiabat(adindex,:,1)>0)); %beginning of adiabat
        rhob(Mcore,adindex,l,i) = ... %local density
            he_density_factor.*interp1(squeeze(my_adiabat(adindex,boa:eoa,1)),squeeze(my_adiabat(adindex,boa:eoa,4)),pr(Mcore,adindex,l,i));
        while pr(Mcore,adindex,l,i) > P_RCB & r < (R_pl * 3) %pressure as a function of radius
            i = i + 1;
            g(Mcore,adindex,l,i) = G*M_pl/((R_pl + r).^2); 
            pr(Mcore,adindex,l,i) = pr(Mcore,adindex,l,i-1) - dr*rhob(Mcore,adindex,l,i-1)*g(Mcore,adindex,l,i);
            r = r + dr;
            rhob(Mcore,adindex,l,i) = ... %local density (this shell of atmosphere)
             he_density_factor.*interp1(squeeze(my_adiabat(adindex,boa:eoa,1)),squeeze(my_adiabat(adindex,boa:eoa,4)),pr(Mcore,adindex,l,i));
            Tb(Mcore,adindex,l,i) = ... %local temperature (this shell of atmosphere)
             interp1(squeeze(my_adiabat(adindex,boa:eoa,1)),T(boa:eoa),pr(Mcore,adindex,l,i));
            mshellb(Mcore,adindex,l,i) = ... %mass in this shell of atmosphere
             4*pi*((R_pl + r).^2)*dr*rhob(Mcore,adindex,l,i-1); %kg
        end
    end
end

end
    
figure;plot(dr*[1:size(mshellb,4)],cumsum(squeeze(mshellb(4,:,end,:))'));
xlabel('Height above volatile-silicate interface (m)')
ylabel('Cumulative volatile mass below this level, convective zone only (kg)')
legend('low Teff','medium Teff','high Teff')
grid on

figure;plot(dr*[1:size(mshellb,4)],(squeeze(Tb(4,:,end,:))'));
xlabel('Height above volatile-silicate interface (m)')
ylabel('Temperature (K)')
grid on

save results_of_parse_eos_small_radius_exoplanets_for_fugacity_crisis_raw

%process results

mass_atm = sum(mshellb,4);
Tb(Tb==0) = NaN;
boaT = nanmax(Tb,[],4); %bottom-of-atmosphere Temperature
boaT = squeeze(boaT(end,:,:)); %For fixed pressure and T(P=RCB), for fixed P_RCB, the boaT is independent of planet mass.

save results_of_parse_eos_small_radius_exoplanets_for_fugacity_crisis_processed ...
    mass_atm Mcore_list boep_list Tinit_list boaT
